<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 *
 *  @author    EveryPay
 *  @copyright since 2015 EveryPay
 *  @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class EveryPayPaymentModuleFrontController extends ModuleFrontController
{
    public function postProcess()
    {
        $cart = $this->context->cart;
        if ($cart->id_customer == 0 ||
            $cart->id_address_delivery == 0 ||
            $cart->id_address_invoice == 0 ||
            !$this->module->active
        ) {
            $this->sendResponse([], true);
        }

        // Check that this payment option is still available in case the
        // customer changed his address just before the end of the checkout process
        $authorized = false;
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] === 'everypay') {
                $authorized = true;
                break;
            }
        }
        if (!$authorized) {
            $this->sendResponse(
                [
                    'message' => $this->module->l('This payment method is not available.', 'validation'),
                ],
                true
            );
        }

        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            $this->sendResponse([], true);
        }

        if ($this->module->getConfig('create_order') !== 'before') {
            return;
        }

        $currency = $this->context->currency;
        $total = (float)$cart->getOrderTotal(true, Cart::BOTH);
        $customerWasLogged = $this->context->customer->isLogged(); // validateOrder() sets to 0, restore this later

        $this->module->validateOrder(
            $cart->id,
            Configuration::get('PS_OS_EVERYPAY'),
            $total,
            $this->module->getFormattedPaymentMethod(Tools::getValue('payment_method') ?
                Tools::getValue('payment_method') : 'missing_payment_method'),
            null,
            [],
            (int)$currency->id,
            false,
            $customer->secure_key
        );

        $this->context->customer->logged = $customerWasLogged;
    }

    public function initContent()
    {
        parent::initContent();

        include $this->module->getLocalPath() . 'lib/everypay_library.php';

        if ($this->module->getConfig('create_order') !== 'before') {
            $cart = $this->context->cart;
            $orderRef = $cart->id;
            $orderAmount = Tools::ps_round((float)$cart->getOrderTotal(true, Cart::BOTH), 2);
        } elseif ($this->module->currentOrder) {
            $order = new Order((int)$this->module->currentOrder);
            $orderRef = $order->reference;
            $orderAmount = Tools::ps_round($order->total_paid, 2);
        }

        if (!$this->module->currentOrder && $this->module->getConfig('create_order') === 'before') {
            $this->sendResponse([], true);
        }

        $addressDelivery = new Address($this->context->cart->id_address_delivery, $this->context->language->id);
        $addressInvoice = new Address($this->context->cart->id_address_invoice, $this->context->language->id);

        if ($this->module->isLiveMode()) {
            $url = EveryPay::ONE_OFF_PROD_URL;
        } else {
            $url = EveryPay::ONE_OFF_TEST_URL;
        }

        $apiUsername = $this->module->getCredentialSetting('api_username');
        $apiSecret = $this->module->getCredentialSetting('shared_secret');

        $everypayLibrary = new EveryPayLibrary();
        $everypayLibrary->init($apiUsername, $apiSecret);

        $queryFields = [
            'amount' => $orderAmount,
            'account_name' => Configuration::get($this->module->formatName('account_id')),
            'billing_city' => $addressInvoice->city,
            'billing_country' => Country::getIsoById((int)$addressInvoice->id_country),
            'billing_state' => Country::getNameById(Context::getContext()->language->id,
                (int)$addressInvoice->id_country),
            'billing_postcode' => $addressInvoice->postcode,
            'billing_line1' => $addressInvoice->address1,
            'billing_line2' => $addressInvoice->address2 ? $addressInvoice->address2 : "-",
            'billing_line3' => '-',
            'customer_url' => $this->context->link->getModuleLink('everypay', 'customerReturn', array(), true),
            'shipping_city' => $addressDelivery->city,
            'shipping_country' => Country::getIsoById((int)$addressDelivery->id_country),
            'shipping_code' => $addressDelivery->postcode,
            'shipping_line1' => $addressDelivery->address1,
            'shipping_line2' => $addressDelivery->address2 ? $addressDelivery->address2 : "-",
            'shipping_line3' => '',
            'email' => $this->context->customer->email,
            'order_reference' => $orderRef,
            'customer_ip' => $_SERVER['REMOTE_ADDR'],
            'mobile_payment' => true,
        ];

        $data = $everypayLibrary->getFields(
            $queryFields,
            Tools::strtolower($this->context->language->iso_code)
        );

        $rawResponse = $this->sendPostRequestWithBasicAuth(
            $url,
            $data,
            $everypayLibrary->getAPIUsername(),
            $everypayLibrary->getAPISecret()
        );

        if (EveryPay::DEBUG_LOGGING) {
            $this->module->logError(array('response' => $rawResponse, 'payload' => $data), 'Validate::sendPaymentRequest() - json');
        }

        $decodedResponse = json_decode($rawResponse, true);
        $response = [
            'accountName' => $decodedResponse['account_name'],
            'apiUsername' => $decodedResponse['api_username'],
            'bearerToken' => $decodedResponse['mobile_access_token'],
            'orderReference' => $orderRef,
            'paymentLink' => $decodedResponse['payment_link'],
            'returnURL' => $decodedResponse['customer_url'],
            'paymentReference' => $decodedResponse['payment_reference'],
        ];

        $this->sendResponse($response);
    }

    private function sendPostRequestWithBasicAuth($url, $data, $username, $password)
    {
        $request_body = json_encode($data);
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLINFO_HEADER_OUT, true);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $request_body);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Content-Type: application/json',
            'Accept: application/json',
            'Content-Length: ' . strlen($request_body)
        ]);
        curl_setopt($ch, CURLOPT_USERPWD, $username . ":" . $password);
        $response = curl_exec($ch);
        curl_close($ch);

        return $response;
    }

    private function sendResponse($data, $error = false) {
        header('Content-Type: application/json; charset=utf-8');

        $response = [
            'error' => $error,
            'data' => $data,
        ];

        die(json_encode($response));
    }
}
